"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebConsoleArchiveCommandHandler = void 0;
const node_util_1 = require("node:util");
const node_url_1 = require("node:url");
const node_zlib_1 = require("node:zlib");
const node_stream_1 = require("node:stream");
const node_http_1 = require("node:http");
const node_https_1 = require("node:https");
const logger_1 = require("@jfrog-pipelines/logger");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const errors_1 = require("@jfrog-pipelines/errors");
const StepServiceRepository_1 = require("../acl/stepService/StepServiceRepository");
const streams_1 = require("../streams");
const StepConsoleRepository_1 = require("./StepConsoleRepository");
const StepletConsoleRepository_1 = require("./StepletConsoleRepository");
const pipelineAsync = (0, node_util_1.promisify)(node_stream_1.pipeline);
const FILE_NAME_TEMPLATE = 'step_%s_consoles.gz';
const BEARER_AUTHORIZATION_PREFIX = 'Bearer ';
const HTTP_PROTOCOL = 'http';
const REQUEST_PROTOCOL_DELIMITER = '://';
const SUPPORTED_PROTOCOLS = new Set(['http', 'https']);
const RETRY_OPTIONS = { isArchived: false, retryAttemptLeft: 3 };
class WebConsoleArchiveCommandHandler {
    constructor() {
        this.logger = logger_1.LoggerFactory.getLogger('WebConsoleArchiveCommandHandler');
        this.stepServiceRepository = new StepServiceRepository_1.StepServiceRepository();
        this.stepConsoleRepository = new StepConsoleRepository_1.StepConsoleRepository();
        this.stepletConsoleRepository = new StepletConsoleRepository_1.StepletConsoleRepository();
    }
    async handle(command) {
        const startTime = Date.now();
        const eventData = {
            domain: 'log-service',
            eventType: 'onArchive',
            stepId: command.getId(),
            status: 'success',
            durationMillis: 0
        };
        try {
            const stepId = command.getId();
            const archived = await this.handleStepArchivalWithRetry(stepId);
            if (archived) {
                eventData.stepConsoleCount = await this.deleteStepConsoleByStepId(stepId);
                eventData.stepletConsoleCount = await this.deleteStepletArchivedConsole(stepId);
            }
        }
        catch (error) {
            eventData.status = 'failed';
            throw error;
        }
        finally {
            const stopTime = Date.now();
            const timeDiff = new Date(stopTime - startTime);
            eventData.durationMillis = timeDiff.valueOf();
            logger_1.LoggerFactory.logEvent(eventData);
        }
    }
    async archiveStep(stepId) {
        const artifactName = (0, node_util_1.format)(FILE_NAME_TEMPLATE, stepId);
        const artifactoryUrls = (await this.stepServiceRepository.getArtifactUrlByStepId(stepId, artifactName));
        const { put: putRequestUrl, putOpts } = artifactoryUrls;
        const artifactoryToken = this.extractBearerToken(putOpts);
        const protocol = this.getProtocol(putRequestUrl);
        const request = protocol.toLowerCase() === HTTP_PROTOCOL ? node_http_1.request : node_https_1.request;
        const writable = request(this.getArtifactoryRequestOptions(putRequestUrl, artifactoryToken));
        const databaseStream = await this.stepConsoleRepository.getByStepId(stepId);
        const zipStream = (0, node_zlib_1.createGzip)();
        try {
            await pipelineAsync(databaseStream, new streams_1.DbRowTransformer(), zipStream, writable);
            databaseStream.destroy();
        }
        catch (error) {
            this.logger.error('archiveStep failed with', (0, node_util_1.inspect)(error));
            throw error;
        }
        finally {
            this.stepConsoleRepository.releaseConnection();
        }
    }
    getProtocol(requestUrl) {
        if (requestUrl.length === 0) {
            throw new errors_1.ApplicationError(`Invalid requestUrl ${requestUrl}`);
        }
        const protocol = requestUrl.split(REQUEST_PROTOCOL_DELIMITER)[0];
        if (!SUPPORTED_PROTOCOLS.has(protocol.toLowerCase())) {
            throw new errors_1.ApplicationError(`Invalid protocol ${protocol} from getArtifactUrlByStepId`);
        }
        return protocol;
    }
    async deleteStepletArchivedConsole(stepId) {
        const steplets = await this.getStepletsByStepId(stepId);
        const stepletIdsToDelete = steplets.filter((steplet) => steplet.isConsoleArchived).map((steplet) => steplet.id);
        const deleteStepletList = [];
        for (const stepletId of stepletIdsToDelete) {
            deleteStepletList.push(this.deleteStepletConsoleByStepletId(stepletId));
        }
        const metricCountArray = await Promise.all(deleteStepletList);
        let result = 0;
        for (const metricCount of metricCountArray) {
            result = result + metricCount;
        }
        return result;
    }
    async handleStepArchivalWithRetry(stepId, options = RETRY_OPTIONS) {
        let archived = false;
        try {
            if (options.retryAttemptLeft === 0) {
                return false;
            }
            if (!options.isArchived) {
                const stepArchived = await this.isStepArchived(stepId);
                if (stepArchived) {
                    return true;
                }
            }
            await this.archiveStep(stepId);
            archived = true;
            await this.stepServiceRepository.updateStep(stepId, { isConsoleArchived: true });
            return true;
        }
        catch {
            return this.handleStepArchivalWithRetry(stepId, {
                isArchived: archived,
                retryAttemptLeft: options.retryAttemptLeft - 1
            });
        }
    }
    async isStepArchived(stepId) {
        const step = await this.stepServiceRepository.getStepById(stepId);
        return step.isConsoleArchived;
    }
    async deleteStepConsoleByStepId(stepId) {
        const deleteQuery = new data_access_layer_1.SequelizeQueryBuilder({}).where('stepId', stepId).build();
        return this.stepConsoleRepository.delete(deleteQuery);
    }
    async getStepletsByStepId(stepId) {
        const steplet = await this.stepServiceRepository.getStepletsByStepId(stepId);
        return steplet;
    }
    async deleteStepletConsoleByStepletId(stepletId) {
        const deleteQuery = new data_access_layer_1.SequelizeQueryBuilder({}).where('stepletId', stepletId).build();
        return this.stepletConsoleRepository.delete(deleteQuery);
    }
    extractBearerToken(header = '') {
        const options = header.replace(/'/g, '');
        const optionsSegmented = options.split(BEARER_AUTHORIZATION_PREFIX);
        return optionsSegmented[1] || '';
    }
    getArtifactoryRequestOptions(requestUrl, artifactoryToken) {
        const requestHeaders = {
            Authorization: `${BEARER_AUTHORIZATION_PREFIX}${artifactoryToken}`,
            'Content-Encoding': 'gzip'
        };
        const parsedUrl = new node_url_1.URL(requestUrl);
        const { hostname, pathname, port } = parsedUrl;
        const options = {
            host: hostname,
            path: pathname,
            method: 'PUT',
            headers: requestHeaders
        };
        if (port) {
            options.port = Number.parseInt(port, 10);
        }
        return options;
    }
}
exports.WebConsoleArchiveCommandHandler = WebConsoleArchiveCommandHandler;
//# sourceMappingURL=WebConsoleArchiveHandler.js.map